#!/bin/sh

MAX_ATTEMPTS="5"
RETRY="15"

function askEmptyValue() {
	read -p "${1}: " VALUE
	echo ${VALUE}
}

function askValue() {
	VALUE=""
	while [[ ${VALUE} == "" ]]
	do
		if [[ -n "${2}" ]]
		then
        		read -p "${1} [${2}]: " VALUE
        		VALUE=${VALUE:-${2}}
		else
			read -p "${1}: " VALUE
		fi
		if [[ -n "${3}" ]]
		then
			VALID=false
			IFS=',' read -ra array <<< "${3}"
			for VALID_VALUE in "${array[@]}"
			do
				if [[ ${VALUE} == ${VALID_VALUE} ]]
				then
					VALID=true
				fi
			done
			if [[ "${VALID}" != "true" ]]
			then
				VALUE=""
			fi
		fi
	done
	echo ${VALUE}
}

function askNumeric() {
	VALUE=""
	while [[ true ]]
	do
        	while [[ ! ${VALUE} =~ ^[0-9]+$ ]]
        	do	
			VALUE=$(askValue "${1}" "${2}")
		done
		if [[ ((-n "${3}") && ("${VALUE}" -lt "${3}")) || ((-n "${4}") &&("${VALUE}" -gt "${4}")) ]]
        	then
                	VALUE=""
        	else
                	break
        	fi
	done
	echo ${VALUE}
}

function isNumeric() {
	[[ ${1} =~ ^[0-9]+$ ]] && echo "true" || echo "false"
}

function getTenantId() {
	TENANT_ESCAPED=$(echo ${1} | sed 's/ /%20/g')
	TENANTID=$(curl -k -q --stderr /dev/null -u ${USERNAME}:${PASSWORD} "http://${HOSTNAME}:${PORT}/pc/center/webservice/tenants/tenantName/${TENANT_ESCAPED}/tenantItemId")
	echo ${TENANTID}
}

function createSNMPProfile() {
	echo "
	<SnmpProfile xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">
        	<name>${1}</name>
        	<communityString>${2}</communityString>
        	<port>${3}</port>
        	<version>${4}</version>
        	<enabled>${5}</enabled>
        	<useForSnmpSet>${6}</useForSnmpSet>
        	<tenantID>${7}</tenantID>
	</SnmpProfile>
	" > .data.xml

	RESULT=$(curl -k -q -XPOST -H "Content-Type: application/xml" --data @.data.xml --stderr /dev/null -u ${USERNAME}:${PASSWORD} "http://${HOSTNAME}:${PORT}/pc/center/webservice/profiles/saveProfile/rankTiesAscendingByDate/")
	echo ${RESULT} | grep -q "<result>true</result>"
	
	echo $?
}

function createDiscProfile() {
	echo "
	<DiscoveryProfile version=\"1.0.0\">
  	  <IcmpDiscoveryEnabled>true</IcmpDiscoveryEnabled>
 	  <DeviceNameRankingList>
            <DeviceNameRanking>{http://im.ca.com/inventory}Device.HostName</DeviceNameRanking>
            <DeviceNameRanking>{http://im.ca.com/inventory}ManageableDevice.SystemName</DeviceNameRanking>
            <DeviceNameRanking>{http://im.ca.com/inventory}Device.PrimaryIPAddress</DeviceNameRanking>
          </DeviceNameRankingList>
          <IPListList>
	" > .data.xml

	if [[ "${5}" != "empty" ]]
	then
		IFS=',' read -ra IPLIST <<< "${5}"
		for IP in "${IPLIST[@]}"
		do
		echo "
	    	<IPList>${IP}</IPList>
		" >> .data.xml
		done
	fi

	echo "
          </IPListList>
	  <IPRangesList>
	" >> .data.xml

	if [[ "${6}" != "empty" ]]
	then
		IFS=',' read -ra IPRANGES <<< "${6}"
		for IPRANGE in "${IPRANGES[@]}"
		do
		echo "
            	<IPRanges>${IPRANGE}</IPRanges>
		" >> .data.xml
		done
	fi

	echo "
	  </IPRangesList>
          <SNMPProfileIDList>
            <SNMPProfileID>${4}</SNMPProfileID>
          </SNMPProfileIDList>
          <CreatePingables>true</CreatePingables>
          <UseListOfSnmpProfiles>true</UseListOfSnmpProfiles>
          <IPDomainMember version=\"1.0.0\">
            <IPDomainID>${3}</IPDomainID>
          </IPDomainMember>
          <Item version=\"1.0.0\">
            <Name>${2}</Name>
          </Item>
        </DiscoveryProfile>
	" >> .data.xml

	RESULT=$(curl -k -q -XPOST -H "Content-Type: application/xml" --data @.data.xml --stderr /dev/null "http://${DAHOSTNAME}:${DAPORT}/rest/tenant/${1}/discoveryprofiles/")
	ID=$(echo "${RESULT}" | awk 'match($0, /<success>(.*)<\/success>/) { print substr($0, RSTART+9, RLENGTH-19) }')

	if [[ "$(isNumeric ${ID})" != "true" ]]
        then
                echo "0"
        else
                echo "${ID}"
        fi

}

function searchIdInDA() {
	echo "
	<FilterSelect xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"filter.xsd\">
  		<Filter><${1}.${2} type=\"EQUAL\">${3}</${1}.${2}></Filter>
		<Select use=\"exclude\" isa=\"exclude\">
   		<${1} use=\"exclude\"></${1}>
  		</Select>
	</FilterSelect>
	" > .data.xml

	ATTEMPT=0
	while [ "${ATTEMPT}" -lt "${MAX_ATTEMPTS}" ]
	do
        	RESULT=$(curl -k -q -XPOST -H "Content-Type: application/xml" --data @.data.xml --stderr /dev/null "http://${DAHOSTNAME}:${DAPORT}${4}")
        	ID=$(echo "${RESULT}" | awk 'match($0, /<ID>(.*)<\/ID>/) { print substr($0, RSTART+4, RLENGTH-9) }')
        	if [[ "${ID}" =~ ^[0-9]+$ ]]
        	then
                	break
        	fi
        	ATTEMPT=`expr ${ATTEMPT} + 1`
        	sleep ${RETRY}
	done

	if [[ "$(isNumeric ${ID})" != "true" ]]
	then
		echo "0"
	else
		echo "${ID}"
	fi
}

echo "Step 1 - Environment"
echo
HOSTNAME=$(askValue "PC Hostname")
PORT=$(askNumeric "PC Port" "8181" "0" "65536")
USERNAME=$(askValue "PC Username" "admin") 
PASSWORD=$(askValue "PC Password")
DAHOSTNAME=$(askValue "DA Hostname")
DAPORT=$(askNumeric "DA Port" "8581" "0" "65536")

echo
echo "Step 2 - SNMP Profile creation in PC"
echo
TENANT=$(askValue "Tenant Name" "Default Tenant")
TENANTID=$(getTenantId "${TENANT}")
if [[ "$(isNumeric $TENANTID)" != "true" ]]
then
	echo "Tenant ${TENANT} not found"
        exit 1
fi
echo "Tenant Id in PC: ${TENANTID}"

SNMP_PROFILE_NAME=$(askValue "SNMP Profile Name")
SNMP_PROFILE_PORT=$(askNumeric "SNMP Profile Port" "161" "0" "65536")
SNMP_PROFILE_VERSION=$(askValue "SNMP Profile version" "Version1or2C" "Version1or2C")
SNMP_PROFILE_COMM=$(askValue "SNMP Profile community" "public")
SNMP_PROFILE_ENABLED=$(askValue "SNMP Profile enabled" "true" "true,false")
SNMP_PROFILE_SET=$(askValue "SNMP Profile valid for set" "true" "true,false")
echo

RESULT=$(createSNMPProfile "${SNMP_PROFILE_NAME}" "${SNMP_PROFILE_COMM}" "${SNMP_PROFILE_PORT}" "${SNMP_PROFILE_VERSION}" "${SNMP_PROFILE_ENABLED}" "${SNMP_PROFILE_SET}" "${TENANTID}")
if [[ "${RESULT}" -eq "0" ]]
then
        echo "Profile created in PC"
else
        echo "Error creating profile in PC"
        exit 1
fi

echo
echo "Step 3 - Discovery Profile creation in DA"
echo
DISC_PROFILE_NAME=$(askValue "Discovery Profile Name")
DOMAIN=$(askValue "Domain Name" "Default Domain")
IPLIST=$(askValue "Comma-separated list of IPs" "empty")
IPRANGES=$(askValue "Comma-separated list of IP Ranges" "empty") 

echo "Searching Tenant Id in DA. Waiting for sync..."
TENANTID=$(searchIdInDA "Tenant" "Name" "${TENANT}" "/rest/tenants/filtered/")
if [[ "${TENANTID}" -eq "0" ]]
then
        echo "Tenant ${TENANT} not found in DA"
        exit 1
fi
echo "Tenant Id in DA: ${TENANTID}"

echo "Searching IP Domain Id in DA. Waiting for sync..."
DOMAINID=$(searchIdInDA "IPDomain" "Name" "${DOMAIN}" "/rest/ipdomains/filtered/")
if [[ "${DOMAINID}" -eq "0" ]]
then
        echo "IP Domain ${DOMAIN} not found in DA"
        exit 1
fi
echo "IP Domain Id in DA: ${DOMAINID}"

echo "Searching SNMP Profile Id in DA. Waiting for sync..."
PROFILEID=$(searchIdInDA "CommunicationProfile" "ProfileName" "${SNMP_PROFILE_NAME}" "/rest/profiles/snmpv1/filtered/")
if [[ "${PROFILEID}" -eq "0" ]]
then
        echo "SNMP Profile ${SNMP_PROFILE_NAME} not found in DA"
        exit 1
fi
echo "SNMP Profile Id in DA: ${PROFILEID}"
echo

DISCPROFILEID=$(createDiscProfile "${TENANTID}" "${DISC_PROFILE_NAME}" "${DOMAINID}" "${PROFILEID}" "${IPLIST}" "${IPRANGES}")
if [[ "${DISCPROFILEID}" -eq "0" ]]
then
	echo "Error creating Discovery Profile in DA"
	exit 1
else
        echo "Discovery Profile created in DA: ${DISCPROFILEID}"
fi
